# Project information

project = "GooseHDF5"
copyright = "2017-2022, Tom de Geus"
author = "Tom de Geus"

# General configuration

extensions = ["sphinx.ext.imgmath", "sphinx.ext.autodoc", "sphinx.ext.autosummary", "sphinxarg.ext"]

exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]

# Options for HTML output

html_theme = "sphinx_rtd_theme"



import os
import subprocess

import h5py
import numpy as np


def run(cmd):
    out = list(filter(None, subprocess.check_output(cmd, shell=True).decode("utf-8").split("\n")))
    return out


a = np.random.random(1000)
b = "foo"

with h5py.File("a.hdf5", "w") as source:
    source["/a"] = a
    source["/b"] = b

output = run("G5repack -c a.hdf5")

with h5py.File("a.hdf5", "r") as source:
    a_r = source["/a"][...]
    b_r = source["/b"].asstr()[...]

os.remove("a.hdf5")

if not np.allclose(a, a_r):
    raise OSError("Test failed")

if b != b_r:
    raise OSError("Test failed")



"""Compare two HDF5 files.
    If the function does not output anything all datasets are present in both files,
    and all the content of the datasets is equal.
    Each output line corresponds to a mismatch between the files.

:usage:

    G5compare [options] <source> <other>

:arguments:

    <source>
        HDF5-file.

    <other>
        HDF5-file.

:options:

    -t, --dtype
        Verify that the type of the datasets match.

    -r, --renamed=ARG
        Renamed paths: this option takes two arguments, one for ``source`` and one for ``other``.
        It can repeated, e.g. ``G5compare a.h5 b.h5 -r /a /b  -r /c /d``

    -c, --color=STR (none, dark)
        Color theme.
        default: dark.

    -h, --help
        Show help.

    --version
        Show version.

(c - MIT) T.W.J. de Geus | tom@geus.me | www.geus.me | github.com/tdegeus/GooseHDF5
"""
import argparse
import os
import warnings

import h5py

from .. import compare_rename
from .. import version

warnings.filterwarnings("ignore")


def theme(theme=None):
    r"""
    Return dictionary of colors.

    .. code-block:: python

        {
            'new' : '...',
            'overwrite' : '...',
            'skip' : '...',
            'bright' : '...',
        }

    :param str theme: Select color-theme.

    :rtype: dict
    """

    if theme == "dark":
        return {
            "->": "9;31",
            "<-": "1;32",
            "!=": "1;31",
        }

    return {
        "->": "",
        "<-": "",
        "!=": "",
    }


class String:
    r"""
    Rich string.

    .. note::

        All options are attributes, that can be modified at all times.

    .. note::

        Available methods:

        *   ``A.format()`` :  Formatted string.
        *   ``str(A)`` : Unformatted string.
        *   ``A.isnumeric()`` : Return if the "data" is numeric.
        *   ``int(A)`` : Dummy integer.
        *   ``float(A)`` : Dummy float.

    :type data: str, None
    :param data: The data.

    :type width: None, int
    :param width: Print width (formatted print only).

    :type color: None, str
    :param color: Print color, e.g. "1;32" for bold green (formatted print only).

    :type align: ``'<'``, ``'>'``
    :param align: Print alignment (formatted print only).

    :type dummy: 0, int, float
    :param dummy: Dummy numerical value.

    :methods:


    """

    def __init__(self, data, width=None, align="<", color=None, dummy=0):

        self.data = data
        self.width = width
        self.color = color
        self.align = align
        self.dummy = dummy

    def format(self):
        r"""
        Return formatted string: align/width/color are applied.
        """

        if self.width and self.color:
            fmt = "\x1b[{color:s}m{{0:{align:s}{width:d}.{width:d}s}}\x1b[0m".format(
                **self.__dict__
            )
        elif self.width:
            fmt = "{{0:{align:s}{width:d}.{width:d}s}}".format(**self.__dict__)
        elif self.color:
            fmt = "\x1b[{color:s}m{{0:{align:s}s}}\x1b[0m".format(**self.__dict__)
        else:
            fmt = "{{0:{align:s}s}}".format(**self.__dict__)

        return fmt.format(str(self))

    def isnumeric(self):
        r"""
        Return if the "data" is numeric : always zero for this class.
        """
        return False

    def __str__(self):
        return str(self.data)

    def __int__(self):
        return int(self.dummy)

    def __float__(self):
        return float(self.dummy)

    def __repr__(self):
        return str(self)

    def __lt__(self, other):
        return str(self) < str(other)


def main():
    class Parser(argparse.ArgumentParser):
        def print_help(self):
            print(__doc__)

    parser = Parser()
    parser.add_argument("-c", "--color", type=str, default="dark")
    parser.add_argument("-t", "--dtype", required=False, action="store_true")
    parser.add_argument("-r", "--renamed", required=False, nargs=2, action="append")
    parser.add_argument("-v", "--version", action="version", version=version)
    parser.add_argument("a")
    parser.add_argument("b")
    args = parser.parse_args()

    color = theme(args.color)

    for filepath in [args.a, args.b]:
        if not os.path.isfile(filepath):
            raise OSError(f'"{filepath}" does not exist')

    with h5py.File(args.a, "r") as a, h5py.File(args.b, "r") as b:
        comp, r_a, r_b = compare_rename(a, b, rename=args.renamed, matching_dtype=args.dtype)

    n = 0
    for key in comp:
        if key != "==":
            for item in comp[key]:
                n = max(n, len(item))

    for item in r_a["=="]:
        n = max(n, len(item))

    paths = []
    for key in comp:
        if key != "==":
            for item in comp[key]:
                paths += [(item, item, key)]

    for path_a, path_b in zip(r_a["!="], r_b["!="]):
        paths += [(path_a, path_b, "!=")]

    paths = sorted(paths, key=lambda v: v[0])

    for path_a, path_b, key in paths:
        if key == "<-":
            print(" " * n + " <- " + String(path_b, color=color["<-"]).format())
        elif key == "->":
            print(String(path_a, color=color["->"], width=n).format() + " -> ")
        else:
            print(
                String(path_a, color=color["!="], width=n).format()
                + " != "
                + String(path_b, color=color["!="], width=n).format()
            )


if __name__ == "__main__":

    main()



import h5py

data = h5py.File("sim1/test.hdf5", "w")

data["/data"] = [1, 2, 3, 4]
data["/tom/data"] = [1, 2, 3, 4]

data.close()

data = h5py.File("sim2/test.hdf5", "w")

data["/data"] = [1, 2, 3, 4]
data["/tom/data"] = [1, 2, 3, 4]

data.close()



import os
import shutil
import unittest

import h5py
import numpy as np

import GooseHDF5 as g5


class Test_itereator(unittest.TestCase):
    """
    Tests.
    """

    def test_getdatasets(self):

        dirname = "mytest"
        filename = "foo.h5"
        filepath = os.path.join(dirname, filename)

        if not os.path.isdir(dirname):
            os.makedirs(dirname)

        Datasets = ["/a", "/b/foo", "/c/d/foo"]

        with h5py.File(filepath, "w") as file:

            for d in Datasets:
                file[d] = [0, 1, 2]

            paths = list(g5.getdatasets(file))
            paths_c = list(g5.getdatasets(file, root="/c"))

        self.assertEqual(sorted(Datasets), sorted(paths))
        self.assertEqual(sorted([Datasets[-1]]), sorted(paths_c))

        shutil.rmtree(dirname)

    def test_getgroups(self):

        dirname = "mytest"
        filename = "foo.h5"
        filepath = os.path.join(dirname, filename)

        if not os.path.isdir(dirname):
            os.makedirs(dirname)

        Datasets = ["/a", "/b/foo", "/c/d/foo"]

        with h5py.File(filepath, "w") as file:

            for d in Datasets:
                file[d] = [0, 1, 2]

            self.assertEqual(g5.getgroups(file), ["/b", "/c", "/c/d"])
            self.assertEqual(g5.getgroups(file, root="/c"), ["/c/d"])

        shutil.rmtree(dirname)

    def test_getgroups_attrs(self):

        dirname = "mytest"
        filename = "foo.h5"
        filepath = os.path.join(dirname, filename)

        if not os.path.isdir(dirname):
            os.makedirs(dirname)

        Datasets = ["/a", "/b/foo", "/c/d/foo"]

        with h5py.File(filepath, "w") as file:

            for d in Datasets:
                file[d] = [0, 1, 2]

            meta = file.create_group("meta")
            meta.attrs["version"] = 0

            self.assertEqual(g5.getgroups(file, has_attrs=True), ["/meta"])

        shutil.rmtree(dirname)

    def test_compare(self):

        dirname = "mytest"
        filename_a = "foo.h5"
        filename_b = "bar.h5"
        filepath_a = os.path.join(dirname, filename_a)
        filepath_b = os.path.join(dirname, filename_b)

        if not os.path.isdir(dirname):
            os.makedirs(dirname)

        with h5py.File(filepath_a, "w") as source:
            with h5py.File(filepath_b, "w") as other:

                # NumPy array

                a = np.random.random(25)

                source["/a/equal"] = a
                source["/a/ne_data"] = a

                other["/a/equal"] = a
                other["/a/ne_data"] = np.random.random(25)

                # single number

                b = float(np.random.random(1))

                source["/b/equal"] = b
                source["/b/ne_data"] = b

                other["/b/equal"] = b
                other["/b/ne_data"] = float(np.random.random(1))

                # string

                c = "foobar"

                source["/c/equal"] = c
                source["/c/ne_data"] = c

                other["/c/equal"] = c
                other["/c/ne_data"] = "foobar2"

                # attribute

                d = np.random.random(25)

                source["/d/equal"] = d
                source["/d/equal"].attrs["key"] = d
                source["/d/ne_attr"] = d
                source["/d/ne_attr"].attrs["key"] = d

                other["/d/equal"] = d
                other["/d/equal"].attrs["key"] = d
                other["/d/ne_attr"] = d
                other["/d/ne_attr"].attrs["key"] = np.random.random(25)

                # dtyoe

                e = (100.0 * np.random.random(25)).astype(int)

                source["/e/equal"] = e
                source["/e/ne_dtype"] = e

                other["/e/equal"] = e
                other["/e/ne_dtype"] = e.astype(float)

                # dtyoe attribute

                f = (100.0 * np.random.random(25)).astype(int)

                source["/f/equal"] = f
                source["/f/equal"].attrs["key"] = f
                source["/f/ne_dtype_attr"] = f
                source["/f/ne_dtype_attr"].attrs["key"] = f

                other["/f/equal"] = f
                other["/f/equal"].attrs["key"] = f
                other["/f/ne_dtype_attr"] = f
                other["/f/ne_dtype_attr"].attrs["key"] = f.astype(float)

                # attribute (not present)

                meta = source.create_group("/meta")
                meta.attrs["version"] = 0

                # attribute (not equal)

                meta = source.create_group("/meta_ne_attr")
                meta.attrs["version"] = 0

                meta = other.create_group("/meta_ne_attr")
                meta.attrs["version"] = 1

                # attribute (equal)

                meta = source.create_group("/meta_equal")
                meta.attrs["version"] = 1

                meta = other.create_group("/meta_equal")
                meta.attrs["version"] = 1

                # check

                check_all = g5.compare(source, other)
                check_datasets = g5.compare(source, other, attrs=False)

        expected_all = {
            "->": ["/meta"],
            "<-": [],
            "!=": [
                "/a/ne_data",
                "/b/ne_data",
                "/c/ne_data",
                "/d/ne_attr",
                "/e/ne_dtype",
                "/f/ne_dtype_attr",
                "/meta_ne_attr",
            ],
            "==": [
                "/a/equal",
                "/b/equal",
                "/c/equal",
                "/d/equal",
                "/e/equal",
                "/f/equal",
                "/meta_equal",
            ],
        }

        expected_datasets = {
            "->": [],
            "<-": [],
            "!=": [
                "/a/ne_data",
                "/b/ne_data",
                "/c/ne_data",
                "/e/ne_dtype",
            ],
            "==": [
                "/a/equal",
                "/b/equal",
                "/c/equal",
                "/d/equal",
                "/d/ne_attr",
                "/e/equal",
                "/f/equal",
                "/f/ne_dtype_attr",
            ],
        }

        for key in expected_all:
            expected_all[key] = sorted(expected_all[key])

        for key in expected_datasets:
            expected_datasets[key] = sorted(expected_datasets[key])

        for key in check_all:
            check_all[key] = sorted(check_all[key])

        for key in check_datasets:
            check_datasets[key] = sorted(check_datasets[key])

        self.assertEqual(expected_all, check_all)
        self.assertEqual(expected_datasets, check_datasets)

        shutil.rmtree(dirname)


if __name__ == "__main__":

    unittest.main()



import os
import shutil
import unittest

import h5py

import GooseHDF5 as g5


class Test_itereator(unittest.TestCase):
    def test_dump(self):

        dirname = "mytest"
        filepath = os.path.join(dirname, "bar.h5")

        if not os.path.isdir(dirname):
            os.makedirs(dirname)

        A = {
            "foo": {"a": 1, "b": 2},
            "bar": {"a": 3, "b": 4},
        }

        paths = []

        for a in A:
            for b in A[a]:
                paths.append(f"/{a}/{b}")

        with h5py.File(filepath, "w") as file:

            g5.dump(file, A)

            self.assertEqual(sorted(g5.getdatasets(file)), sorted(paths))

            for a in A:
                for b in A[a]:
                    self.assertEqual(file[a][b][...], A[a][b])

        shutil.rmtree(dirname)


if __name__ == "__main__":

    unittest.main()



import os
import subprocess

import h5py
import numpy as np


def run(cmd):
    out = list(filter(None, subprocess.check_output(cmd, shell=True).decode("utf-8").split("\n")))
    return [i.rstrip().replace("\r", "") for i in out]


with h5py.File("a.hdf5", "w") as source:
    with h5py.File("b.hdf5", "w") as other:

        # NumPy array

        a = np.random.random(25)

        source["/a/equal"] = a
        source["/a/not_equal"] = a

        other["/a/equal"] = a
        other["/a/not_equal"] = np.random.random(25)

        # single number

        b = float(np.random.random(1))

        source["/b/equal"] = b
        source["/b/not_equal"] = b

        other["/b/equal"] = b
        other["/b/not_equal"] = float(np.random.random(1))

        # string

        c = "foobar"

        source["/c/equal"] = c
        source["/c/not_equal"] = c

        other["/c/equal"] = c
        other["/c/not_equal"] = "foobar2"

        # alias

        d = np.random.random(25)

        source["/d/equal"] = d
        other["/e/equal"] = d

        # attribute

        f = np.random.random(25)

        source["/f/equal"] = f
        source["/f/equal"].attrs["key"] = f
        source["/f/not_equal"] = f
        source["/f/not_equal"].attrs["key"] = f

        other["/f/equal"] = f
        other["/f/equal"].attrs["key"] = f
        other["/f/not_equal"] = f
        other["/f/not_equal"].attrs["key"] = np.random.random(25)

        # meta (not present)

        meta = source.create_group("/meta")
        meta.attrs["version"] = 0

        # meta (present)

        meta = source.create_group("/present")
        meta.attrs["version"] = 0

        meta = other.create_group("/present")
        meta.attrs["version"] = 1


output = sorted(run("G5compare -c none a.hdf5 b.hdf5 -r /d/equal /e/equal"))

expected_output = sorted(
    [
        "/a/not_equal != /a/not_equal",
        "/b/not_equal != /b/not_equal",
        "/c/not_equal != /c/not_equal",
        "/f/not_equal != /f/not_equal",
        "/present     != /present",
        "/meta        ->",
    ]
)

os.remove("a.hdf5")
os.remove("b.hdf5")

if output != expected_output:
    print("output = ")
    print(output)
    print("expected output = ")
    print(expected_output)
    raise OSError("Test failed")



from setuptools import find_packages
from setuptools import setup

project_name = "GooseHDF5"

setup(
    name=project_name,
    license="MIT",
    author="Tom de Geus",
    author_email="tom@geus.me",
    description="Wrapper around h5py",
    long_description="Wrapper around h5py",
    keywords="HDF5, h5py",
    url=f"https://github.com/tdegeus/{project_name:s}",
    packages=find_packages(),
    use_scm_version={"write_to": f"{project_name}/_version.py"},
    setup_requires=["setuptools_scm"],
    install_requires=["h5py", "click"],
    entry_points={
        "console_scripts": [
            f"G5check = {project_name}.cli.G5check:main",
            f"G5compare = {project_name}.cli.G5compare:main",
            f"G5list = {project_name}:_G5list_catch",
            f"G5print = {project_name}:_G5print_catch",
            f"G5repack = {project_name}.cli.G5repack:main",
            f"G5repair = {project_name}.cli.G5repair:main",
        ]
    },
)



"""Read and write a HDF5 file.
    This allows a compacter file by removing features like extendible datasets.

:usage:

    G5repack [options] <source>...

:arguments:

    <source>
        HDF5-file.

:options:

    -c, --compress
        Apply compression (using the loss-less GZip algorithm).

    -f, --float
        Store doubles as floats.

    -h, --help
        Show help.

    --version
        Show version.

(c - MIT) T.W.J. de Geus | tom@geus.me | www.geus.me | github.com/tdegeus/GooseHDF5
"""
import argparse
import os
import tempfile
import warnings

import h5py

from .. import copy_dataset
from .. import getdatasets
from .. import version

warnings.filterwarnings("ignore")


def check_isfile(fname):
    if not os.path.isfile(fname):
        raise OSError(f'"{fname}" does not exist')


def main():

    try:

        class Parser(argparse.ArgumentParser):
            def print_help(self):
                print(__doc__)

        parser = Parser()
        parser.add_argument("-c", "--compress", required=False, action="store_true")
        parser.add_argument("-f", "--float", required=False, action="store_true")
        parser.add_argument("-v", "--version", action="version", version=version)
        parser.add_argument("source", nargs="+")
        args = parser.parse_args()

        tempname = next(tempfile._get_candidate_names())

        for filename in args.source:

            print(filename)

            check_isfile(filename)

            with h5py.File(filename, "r") as source:
                with h5py.File(tempname, "w") as tmp:
                    copy_dataset(source, tmp, getdatasets(source), args.compress, args.float)

            os.replace(tempname, filename)

    except Exception as e:

        print(e)
        return 1


if __name__ == "__main__":

    main()
